import json

from typing import List, Literal
from pydantic import BaseModel, Field, root_validator, ValidationError

from ieg_validators.cloudport.utils import load_yaml
from ieg_validators.cloudport.errors import InvalidCRD
from ieg_validators.cloudport.models.blip.settings import AutomationSettings
from ieg_validators.cloudport.models.blip.postmaster import PostmasterSubscribers


class Provider(BaseModel):
    externalSecretEnabled: bool
    imagePullSecret: str


class Fluentd(BaseModel):
    endpoint: str
    port: int


class Redis(BaseModel):
    type: Literal["managed", "local"]
    nodeType: str
    numOfReplicas: int


class BlipEnv(BaseModel):
    RECAPTCHA_SITE_KEY: str
    RECAPTCHA_SECRET: str
    PUBNUB_PUBLISH_KEY: str
    PUBNUB_SUBSCRIBE_KEY: str
    PUBNUB_SECRET_KEY: str
    CPLIVE_PUBNUB_PUBLISH_KEY: str | None
    CPLIVE_PUBNUB_SUBSCRIBE_KEY: str | None


class Blip(BaseModel):
    containerImage: str
    setupToken: str
    hostname: str
    env: BlipEnv


class Toleration(BaseModel):
    key: str
    operator: str
    effect: str
    value: str


class NodeSelector(BaseModel):
    role: str


class Infra(BaseModel):
    tolerations: List[Toleration]
    nodeSelector: NodeSelector


class Mon(BaseModel):
    enabled: bool


class Postmaster(BaseModel):
    enabled: bool


class Monitoring(BaseModel):
    containerImage: str


class PostmasterEnv(BaseModel):
    POSTMASTER_LOG_LEVEL: str


class PostmasterConfig(BaseModel):
    containerImage: str
    env: PostmasterEnv | None


class DynamiteEnv(BaseModel):
    SMTP_LOGIN: bool | None
    SMTP_HOST: str | None
    SMTP_PORT: int | None
    SENDER_EMAIL_ID: str | None
    ATTACH_CSV_REPORT: bool | None
    ALERT_RECIPIENTS: str | None
    MEDIA_TYPES_TO_EXCLUDE: str | None
    REDIS_ENDPOINT: str | None
    REDIS_DATABASE: int | None
    EXPIRY_TIME_IN_SEC: int | None
    FLUENTD_ENDPOINT: str | None


class Dynamite(BaseModel):
    enabled: bool
    containerImage: str
    sidecarImage: str
    env: DynamiteEnv


class Resources(BaseModel):
    cpu: str
    memory: str


class SidekiqFeedHpa(BaseModel):
    queues: List[str]
    min: int
    max: int
    pollingInterval: int
    cooldownPeriod: int


class SidekiqFeed(BaseModel):
    provisionedFeeds: int
    resources: Resources
    hpa: SidekiqFeedHpa


class SidekiqHpa(BaseModel):
    queues: List[str]
    pollingInterval: int
    cooldownPeriod: int
    min: int
    max: int


class Sidekiq(BaseModel):
    enabled: bool | None
    replicas: int | None
    resources: Resources
    hpa: SidekiqHpa | None


class WebHpa(BaseModel):
    min: int
    max: int


class Web(BaseModel):
    replicas: int
    resources: Resources
    hpa: WebHpa


class SentinelEnv(BaseModel):
    SKYNET_AUTO_UPDATE: bool
    SKYNET_URL: str
    SKYNET_TOKEN: str
    SKYNET_BLIP_USER: str


class Sentinel(BaseModel):
    enabled: bool
    containerImage: str
    replicas: int
    env: SentinelEnv
    resources: Resources


class Configs(BaseModel):
    postmaster_subscribers_json: PostmasterSubscribers | None = Field(
        alias="postmaster_subscribers.json"
    )
    settings_yml: AutomationSettings = Field(alias="settings.yml")

    @root_validator(pre=True)
    def serialize(cls, value):
        try:
            if value.get("postmaster_subscribers.json"):
                value["postmaster_subscribers.json"] = PostmasterSubscribers(
                    **json.loads(value["postmaster_subscribers.json"])
                )
            value["settings.yml"] = AutomationSettings(
                **load_yaml(value["settings.yml"], "ruamel")
            )
        except ValidationError as e:
            raise InvalidCRD(
                "Invalid json found at settings.yml.postmaster_subscribers in blip CRD",
                e,
            ) from e
        except Exception as e:
            raise e

        return value


class Telescope(BaseModel):
    enabled: bool


class CRDAutomation(BaseModel):
    provider: Provider
    fluentd: Fluentd | None
    redis: Redis
    blip: Blip
    infra: Infra | None
    mon: Mon | None
    postmaster: Postmaster | None
    monitoring: Monitoring | None
    postmaster_config: PostmasterConfig | None
    dynamite: Dynamite | None
    sidekiqFeed: SidekiqFeed | None
    sidekiq: Sidekiq | None
    web: Web | None
    sentinel: Sentinel | None
    configs: Configs
    telescope: Telescope | None

    @root_validator(pre=True)
    def postmaster_config_check(cls, value):
        if (value.get("postmaster") and value["postmaster"]["enabled"]) and (
            not value.get("postmaster_config")
            or not value["configs"].get("postmaster_subscribers.json")
        ):
            raise InvalidCRD("postmaster enabled but config not present in blip CRD")

        return value
